@@ -13,17 +13,17 @@ package ClamTk::App;
 
 use strict;
 #use warnings;    # disabled upon release
+$|++;
 
 use Carp qw/croak cluck/;
 use Net::DNS;
-$|++;
 
 use encoding 'utf8';
 
 sub get_TK_version {
 
     # Stick with %.2f format - 4.50 vice 4.5
-    return '4.25';
+    return '4.26';
 }
 
 sub get_path {
@@ -245,7 +245,8 @@ sub get_sig_info {
                     last;
                 }
             }
-            close($FILE);
+            close($FILE)
+                or warn "Couldn't close $daily_path: $!\n";
         }
         else {
             $INFO_DATE  = '01 Jan 1970';
