@@ -13,10 +13,10 @@ package ClamTk::GUI;
 
 use strict;
 #use warnings;    # disabled upon release
+$|++;
 
 use Carp qw/cluck croak/;
 use encoding 'utf8';
-$|++;
 
 use ClamTk::App qw/get_TK_version get_AV_version get_AV_remote get_date_sigs
     get_path get_num_sigs/;
@@ -616,10 +616,10 @@ sub set_sig_status {
 }
 
 sub set_tk_status {
-    shift;
+    shift;    # throw away package name
     my ($ret) = shift;
 
-    if ( $ret == 0 ) {
+    if ( $ret == 4 || $ret == 5 ) {
         $status_gui_img->set_from_stock( 'gtk-dialog-question',
             'small-toolbar' );
         $tt->set_tip( $status_gui_img, gettext('Unable to check') );
@@ -1273,7 +1273,7 @@ sub restore {
 
     if ($path) {
         if ( -e $path ) {    # another file already exists there
-	    $final_destination = $path .= '.restore';
+            $final_destination = $path .= '.restore';
         }
         else {
             $final_destination = $path;
@@ -1681,7 +1681,7 @@ sub preferences {
 
     my $box = Gtk2::VBox->new( FALSE, 0 );
     $pref->add($box);
-    $box->set_size_request( 510, 300 );
+    $box->set_size_request( 530, 300 );
 
     my $nb = Gtk2::Notebook->new;
     $nb->set_scrollable(TRUE);
@@ -1892,16 +1892,16 @@ sub preferences {
     $man_table->set_row_spacings(5);
 
     my $ip_label   = Gtk2::Label->new( gettext('IP address or host:') );
-    my $ip_example = Gtk2::Label->new('(e.g., http://127.0.0.1)');
+    my $ip_example = Gtk2::Label->new('(e.g., proxy.domain.com)');
 
     # We can't go with max_length(IP address) because they
     # can insert a hostname here as well. 63 should be enough for anyone.
     my $ip_box = Gtk2::Entry->new_with_max_length(63);
-    $ip_box->set_editable(FALSE);
     $ip_box->signal_connect(
         'insert-text' => sub {
             my ( $widget, $string, $position ) = @_;
-	    # http://www.ietf.org/rfc/rfc1738.txt
+
+            # http://www.ietf.org/rfc/rfc1738.txt
             if ( $string !~ m#[a-zA-Z0-9\.\-\+/:\@\#]# ) {
                 $ip_box->signal_stop_emission_by_name('insert-text');
             }
@@ -1910,43 +1910,38 @@ sub preferences {
     );
 
     my $port_label   = Gtk2::Label->new( gettext('Port:') );
-    my $port_example = Gtk2::Label->new('(e.g., 8888)');
-    my $port_box     = Gtk2::Entry->new_with_max_length(6);
-    $port_box->signal_connect(
-        'insert-text' => sub {
-            my ( $widget, $string, $position ) = @_;
-            if ( $string !~ /[0-9]/ ) {
-                $port_box->signal_stop_emission_by_name('insert-text');
-            }
-            return;
-        }
-    );
-    $port_box->set_editable(FALSE);
+    my $port_example = Gtk2::Label->new('(e.g., 8080)');
+    my $port_box     = Gtk2::SpinButton->new_with_range( 1, 65535, 1 );
+    $port_box->set_value(8080);
 
     $man_table->attach_defaults( $ip_label,   0, 1, 0, 1 );
     $man_table->attach_defaults( $ip_example, 1, 2, 0, 1 );
-    $man_table->attach_defaults( $ip_box,     2, 3, 0, 1 );
+    $man_table->attach( $ip_box, 2, 3, 0, 1, [qw/expand expand/],
+        [qw/shrink/], 0, 0 );
 
     $man_table->attach_defaults( $port_label,   0, 1, 1, 2 );
     $man_table->attach_defaults( $port_example, 1, 2, 1, 2 );
-    $man_table->attach_defaults( $port_box,     2, 3, 1, 2 );
+    $man_table->attach( $port_box, 2, 3, 1, 2, [qw/shrink shrink/],
+        [qw/shrink/], 0, 0 );
 
     $proxy_btn->signal_connect(
         toggled => sub {
-            $ip_box->set_editable(FALSE)   if ( $proxy_btn->get_active );
-            $port_box->set_editable(FALSE) if ( $proxy_btn->get_active );
+            $ip_box->set_sensitive(FALSE)   if ( $proxy_btn->get_active );
+            $port_box->set_sensitive(FALSE) if ( $proxy_btn->get_active );
         }
     );
     $sys_btn->signal_connect(
         toggled => sub {
-            $ip_box->set_editable(FALSE)   if ( $sys_btn->get_active );
-            $port_box->set_editable(FALSE) if ( $sys_btn->get_active );
+            $ip_box->set_sensitive(FALSE)   if ( $sys_btn->get_active );
+            $port_box->set_sensitive(FALSE) if ( $sys_btn->get_active );
         }
     );
     $man_btn->signal_connect(
         toggled => sub {
-            $ip_box->set_editable(TRUE)   if ( $man_btn->get_active );
-            $port_box->set_editable(TRUE) if ( $man_btn->get_active );
+            if ( $man_btn->get_active ) {
+                $ip_box->set_sensitive(TRUE);
+                $port_box->set_sensitive(TRUE);
+            }
         }
     );
 
@@ -1971,13 +1966,6 @@ sub preferences {
                 $choice = 0;
             }
             if ( $choice == 0 || $choice == 1 ) {
-                $ip_box->set_text('');
-                $port_box->set_text('');
-                my $path = ClamTk::App->get_path('db');
-                $path .= '/local.conf';
-                if ( -f $path ) {
-                    unlink $path;
-                }
                 if ( ClamTk::Prefs->set_preference( 'HTTPProxy', $choice ) ) {
                     proxy_non_block_status('yes');
                 }
@@ -1991,35 +1979,36 @@ sub preferences {
                     $proxy_btn->set_active(TRUE);
                     return;
                 }
+                my $ip = $ip_box->get_text;
+                if ( $ip !~ m#://# ) {
+                    $ip = 'http://' . $ip;
+                }
+                my $port = $port_box->get_value_as_int;
 
                 # Hate to pull in LWP::UserAgent just for this,
                 # but we need to sanity check it before they get
                 # to using it in the first place
                 eval {
-                    my $ua   = LWP::UserAgent->new;
-                    my $ip   = $ip_box->get_text;
-                    my $port = $port_box->get_text;
+                    my $ua = LWP::UserAgent->new;
                     $ua->proxy( http => "$ip:$port" );
                 };
                 if ($@) {
                     proxy_non_block_status('no');
-                    $ip_box->set_text('');
                     $ip_example->set_markup(
-                        qq(<span underline="error" underline_color="red">(e.g., http://127.0.0.1)</span>)
+                        qq(<span underline="error" underline_color="red">(e.g., proxy.domain.com)</span>)
                     );
                     return;
                 }
-                $ip_example->set_text('(e.g., http://127.0.0.1)');
-                if (ClamTk::Prefs->set_preference( 'HTTPProxy', $choice )
-                    && ClamTk::Prefs->set_proxy(
-                        $ip_box->get_text, $port_box->get_text
-                    )
-                    )
+                $ip_example->set_text('(e.g., proxy.domain.com)');
+                if (   ClamTk::Prefs->set_preference( 'HTTPProxy', $choice )
+                    && ClamTk::Prefs->set_proxy( $ip, $port ) )
                 {
                     proxy_non_block_status('yes');
+                    $ip_box->set_text($ip);
                 }
                 else {
                     proxy_non_block_status('no');
+                    $ip_box->set_text($ip);
                 }
             }
         }
@@ -2031,12 +2020,7 @@ sub preferences {
             $ip_box->set_text('');
             $port_box->set_text('');
             $proxy_btn->set_active(TRUE);
-            $ip_example->set_text('(e.g., http://127.0.0.1)');
-            my $path = ClamTk::App->get_path('db');
-            $path .= '/local.conf';
-            if ( -f $path ) {
-                unlink $path;
-            }
+            $ip_example->set_text('(e.g., proxy.domain.com)');
             if ( ClamTk::Prefs->set_preference( 'HTTPProxy', 0 ) ) {
                 proxy_non_block_status('yes');
             }
@@ -2053,33 +2037,47 @@ sub preferences {
 
     # What does the user have set?
     # 0 = no proxy, 1 = env_proxy and 2 = manual proxy
-    if ( ClamTk::Prefs->get_preference('HTTPProxy') ) {
-        if ( ClamTk::Prefs->get_preference('HTTPProxy') == 1 ) {
-            $sys_btn->set_active(TRUE);
-        }
-        elsif ( ClamTk::Prefs->get_preference('HTTPProxy') == 2 ) {
-            $man_btn->set_active(TRUE);
-            my $path = ClamTk::App->get_path('db');
-            $path .= '/local.conf';
-            if ( -f $path ) {
-                if ( open( my $FH, '<', $path ) ) {
-                    while (<$FH>) {
-                        chomp;
-                        if (/HTTPProxyServer\s+(.*?)$/) {
-                            $ip_box->set_text($1);
-                        }
-                        if (/HTTPProxyPort\s+(.*?)$/) {
-                            $port_box->set_text($1);
-                        }
+
+    $ip_box->set_sensitive(FALSE);
+    $port_box->set_sensitive(FALSE);
+
+    my $setting = ClamTk::Prefs->get_preference('HTTPProxy');
+    if ( !$setting ) {
+        $proxy_btn->set_active(TRUE);
+    }
+    elsif ( $setting == 1 ) {
+        $sys_btn->set_active(TRUE);
+    }
+    elsif ( $setting == 2 ) {
+        $man_btn->set_active(TRUE);
+    }
+    my $path = ClamTk::App->get_path('db');
+    $path .= '/local.conf';
+    if ( -f $path ) {
+        if ( open( my $FH, '<', $path ) ) {
+            while (<$FH>) {
+                chomp;
+                my $set;
+                if (/HTTPProxyServer\s+(.*?)$/) {
+                    $set = $1;
+                    if ( $set !~ m#://# ) {
+                        $set = 'http://' . $set;
+                    }
+                    $ip_box->set_text($set);
+                    if ( !$setting || $setting == 1 ) {
+                        $ip_box->set_sensitive(FALSE);
+                    }
+                }
+                if (/HTTPProxyPort\s+(.*?)$/) {
+                    $port_box->set_value($1);
+                    if ( !$setting || $setting == 1 ) {
+                        $port_box->set_sensitive(FALSE);
                     }
-                    close($FH);
                 }
             }
+            close($FH);
         }
     }
-    else {
-        $proxy_btn->set_active(TRUE);
-    }
 
     $box->pack_start( $nb, TRUE, TRUE, 0 );
 
