@@ -37,9 +37,9 @@ my $slist;
 my $hover_label;
 
 sub display {
-    shift;    # Don't need package name
-    my ($cmd_scan) = shift;
-    my ($hash)     = @_;
+    shift;    # don't need package name
+    my $cmd_scan = shift;
+    my ($hash) = @_;
 
     $popup = Gtk2::Window->new();
     $popup->signal_connect( destroy => sub { $popup->destroy } );
@@ -69,7 +69,9 @@ sub display {
     );
     $scrolled_win->add($slist);
     $scrolled_win->grab_focus();
-    $slist->get_selection->set_mode('single');
+
+    #$slist->get_selection->set_mode('single');
+    $slist->get_selection->set_mode('multiple');
     $slist->set_rules_hint(TRUE);
     $slist->set_headers_clickable(TRUE);
 
@@ -123,20 +125,15 @@ sub display {
             my $quar_image =
                 Gtk2::Image->new_from_stock( 'gtk-refresh', 'menu' );
             $quar_pop->set_image($quar_image);
-            if ( $hash->{$deref}->{name} =~ /$maildirs/ ) {
-                $hover_label->set_text(
-                    gettext(
-                        'Possible email file - please manually quarantine or delete it.'
-                    )
-                );
-            }
-            elsif ( -d $hash->{$deref}->{name} ) {
-                return;
-            }
-            else {
-                $quar_pop->signal_connect(
-                    activate => sub { main_confirm( $deref, 'q' ) } );
-            }
+            $quar_pop->signal_connect(
+                activate => sub {
+                    for my $t (@sel) {
+                        next if ( $hash->{$t}->{name} =~ /$maildirs/ );
+                        next if ( -d $hash->{$t}->{name} );
+                        main_confirm( $t, 'q' );
+                    }
+                }
+            );
             $quar_pop->show();
             $menu->append($quar_pop)
                 unless dirname( $hash->{$deref}->{name} )
@@ -148,20 +145,15 @@ sub display {
             my $del_image =
                 Gtk2::Image->new_from_stock( 'gtk-delete', 'menu' );
             $delete_pop->set_image($del_image);
-            if ( $hash->{$deref}->{name} =~ /$maildirs/ ) {
-                $hover_label->set_text(
-                    gettext(
-                        'Possible email file - please manually quarantine or delete it.'
-                    )
-                );
-            }
-            elsif ( -d $hash->{$deref}->{name} ) {
-                return;
-            }
-            else {
-                $delete_pop->signal_connect(
-                    activate => sub { main_confirm( $deref, 'd' ) } );
-            }
+            $delete_pop->signal_connect(
+                activate => sub {
+                    for my $u (@sel) {
+                        next if ( $hash->{$u}->{name} =~ /$maildirs/ );
+                        next if ( -d $hash->{$u}->{name} );
+                        main_confirm( $u, 'd' );
+                    }
+                }
+            );
             $delete_pop->show();
             $menu->append($delete_pop)
                 unless dirname( $hash->{$deref}->{name} )
@@ -265,6 +257,7 @@ sub main_confirm {
             return;
         }
         my $confirm_message = gettext('Really delete this file?');
+        $confirm_message .= ' ' . '(' . basename($full_name) . ')';
         my $confirm =
             Gtk2::MessageDialog->new( $popup, [qw(modal destroy-with-parent)],
             'question', 'ok-cancel', $confirm_message );
