@@ -215,10 +215,11 @@ sub decision {
                 $update_list->{data}[$row][3] = gettext('Checking...');
                 $value = update_gui( 'dummy', 'not-startup' );
                 $update_list->{data}[$row][3] =
-                      $value == -1 ? gettext('Check failed')
-                    : $value == 1  ? gettext('GUI is current')
+                      $value == 1  ? gettext('GUI is current')
                     : $value == 2  ? gettext('A newer version is available')
                     : $value == 3  ? gettext('GUI is current')
+                    : $value == 4  ? gettext('Check failed')
+                    : $value == 5  ? gettext('Check failed')
                     :                '';
                 ClamTk::GUI->set_tk_status($value);
             }
@@ -350,7 +351,7 @@ sub update_signatures {
 }
 
 sub update_gui {
-    shift;
+    shift; # throw away package name
     my $caller = shift;
 
     my ($version) = ClamTk::App->get_TK_version();
@@ -404,13 +405,14 @@ sub update_gui {
         my $content = $response->content;
         chomp($content);
         $content =~ s/[^0-9]//g;
-        return 1 if ( $version == $content );
-        return 2 if ( $content > $version );
-        return 3 if ( $version > $content );
-        return -1;
+        return 1 if ( $version == $content ); # current
+        return 2 if ( $content > $version );  # outdated
+        return 3 if ( $version > $content );  # too current?
+        return 4;			      # shouldn't happen
     }
     else {
-        return -1;
+	# warn $response->as_string, "\n";
+        return 5;			      # failed, unable to check
     }
 }
 
@@ -458,7 +460,13 @@ sub av_db_select {
     my $c_box = Gtk2::HButtonBox->new();
     $choice->vbox->pack_start( $c_box, FALSE, FALSE, 0 );
 
-    my ( $single_btn, $sys_btn, $save_btn, $quit_btn, $label );
+    my ( $cancel_btn, $single_btn, $sys_btn, $save_btn, $quit_btn, $label );
+
+    $cancel_btn = Gtk2::Button->new_from_stock('gtk-cancel');
+    $c_box->add($cancel_btn);
+    $cancel_btn->signal_connect(
+	clicked => sub { $choice->destroy }
+    );
 
     $single_btn = Gtk2::Button->new( gettext('Single User') );
     $c_box->add($single_btn);
